/*
 * Hibernate, Relational Persistence for Idiomatic Java
 *
 * Copyright (c) 2009, Red Hat Middleware LLC or third-party contributors as
 * indicated by the @author tags or express copyright attribution
 * statements applied by the authors.  All third-party contributions are
 * distributed under license by Red Hat Middleware LLC.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
/**
 * 
 */
package org.hibernate.test.annotations;

import java.io.IOException;
import java.net.URL;
import java.util.Properties;

import org.hibernate.util.ConfigHelper;

/**
 * This util class is for setting some system-level properties It is used before
 * Environment be loaded into jvm.
 * 
 * @author Strong Liu <stliu@redhat.com>
 */
public class SystemLevelPropertyUtil {
	/**
	 * find is the property <tt>prop</tt> is in System properties or in
	 * hibernate.properties.
	 * 
	 * 
	 * @param prop
	 * @return
	 */
	public static boolean isPropertyExist( Object prop ) {
		if ( System.getProperties().containsKey( prop ) ) {
			return true;
		}
		return getHibernateProperties().containsKey( prop );
	}
	
	/**
	 * get the property <tt>prop</tt> value 
	 * @param prop
	 * @return
	 */
	public static Object getSystemLevelProperty( Object prop ) {
		if ( !isPropertyExist( prop ) ) {
			return null;
		}
		if ( System.getProperties().containsKey( prop ) ) {
			return System.getProperties().get( prop );
		}
		if ( getHibernateProperties().containsKey( prop ) ) {
			return getHibernateProperties().get( prop );
		}
		return null;
	}

	private static Properties getHibernateProperties() {
		Properties properties = new Properties();
		try {
			properties.load( findAsResource( "hibernate.properties" )
					.openStream() );
		} catch (IOException e) {

		}
		return properties;
	}

	/**
	 * Try to locate a local URL representing the incoming path. This method
	 * <b>only</b> attempts to locate this URL as a java system resource.
	 * 
	 * @param path
	 *            The path representing the config location.
	 * @return An appropriate URL or null.
	 */
	private static final URL findAsResource( final String path ) {
		URL url = null;

		// First, try to locate this resource through the current
		// context classloader.
		ClassLoader contextClassLoader = Thread.currentThread()
				.getContextClassLoader();
		if ( contextClassLoader != null ) {
			url = contextClassLoader.getResource( path );
		}
		if ( url != null )
			return url;

		// Next, try to locate this resource through this class's classloader
		url = ConfigHelper.class.getClassLoader().getResource( path );
		if ( url != null )
			return url;

		// Next, try to locate this resource through the system classloader
		url = ClassLoader.getSystemClassLoader().getResource( path );

		// Anywhere else we should look?
		return url;
	}
}
